<?php

/**
 * Core plugin class
 *
 * @since      1.0.0
 * @package    TyresAddict
 * @subpackage TyresAddict/Finder/includes
 * @author     TyresAddict
 * @link       http://b2b.tyresaddict.com
 */

namespace TyresAddict\Finder;


/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 */
class Plugin 
{
	const title 	= \TyresAddictWooFinderPluginProVer::title;
	const name 		= \TyresAddictWooFinderPluginProVer::name;
	const lang 		= \TyresAddictWooFinderPluginProVer::lang;
	const code 		= \TyresAddictWooFinderPluginProVer::code;
	const version 	= \TyresAddictWooFinderPluginProVer::version;
	const prefix 	= \TyresAddictWooFinderPluginProVer::name . '-';
	const features 	= \TyresAddictWooFinderPluginProVer::features;


	// The loader that's responsible for maintaining and registering all hooks that power the plugin.
	protected $loader;

	protected $plugin_public 	= null;		// public part of plugin
	protected $plugin_admin 	= null;		// admin part of plugin

	protected $page_options 	= null;		// options page

	protected $_features = [];	

	// singleton

	protected static $instance;

	public static function get_instance() {
		if ( is_null( static::$instance ) ) {
			static::$instance = new static;
			static::$instance->run();			// run loader
		}
		return static::$instance;
	}
	

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 */
	public function __construct() 
	{              
		$this->load_dependencies();

		PluginI18n::textdomains();
		
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - PluginLoader. Orchestrates the hooks of the plugin.
	 * - PluginI18n. Defines internationalization functionality.
	 * - PluginAdmin. Defines all hooks for the admin area.
	 * - PluginPublic. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() 
	{
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/PluginLoader.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/PluginI18n.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/PluginAdmin.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/PluginPublic.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/DB.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/Woo.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/Template.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/PluginOptions.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/PageOptions.php';

		foreach( Plugin::features as $feature )
		{
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/' . $feature . '.php';

			$feature_class = "TyresAddict\\Finder\\" . $feature;
			$this->_features[ $feature ] = new $feature_class();
		}

		$this->loader = new PluginLoader();
	}

	// Register all of the hooks related to the admin area functionality

	private function define_admin_hooks() 
	{
		$this->plugin_admin = new PluginAdmin();

		$this->loader->add_action( 'admin_enqueue_scripts', $this->plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $this->plugin_admin, 'enqueue_scripts' );

		$this->page_options = new PageOptions();
	}


	// Register all of the hooks related to the public-facing functionality

	private function define_public_hooks() 
	{
		$this->plugin_public = new PluginPublic();

		add_action( 'wp_ajax_tyresaddict_woo_finder', [ $this->plugin_public, 'ajax_controller' ] );
		add_action( 'wp_ajax_nopriv_tyresaddict_woo_finder', [ $this->plugin_public, 'ajax_controller' ] );

		$this->loader->add_action( 'wp_enqueue_scripts', $this->plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $this->plugin_public, 'enqueue_scripts' );
	}


	public function run() 
	{
		$this->loader->run();

		foreach( Plugin::features as $feature )
		{
		    if ( method_exists( $this->_features[ $feature ], 'run' ) )
				$this->_features[ $feature ]->run();
		}
	}

	public function get_loader() {
		return $this->loader;
	}

	static function path() {
		return plugin_dir_path( dirname( __FILE__ ) );
	}
}

